////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// HANK_q4.mod
//
// October 2021
//
// Replication codes: "Monetary Policy and Capital Misallocation"
//
// (c) Beatriz Gonzalez, Galo Nuno, Dominik Thaler and Silvia Albrizio
//
// This is the main file to solve for the optimal monetary policy in baseline economy
//
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// prepare model definition

@#define N = 200

var Omega phi w r A X K D Z C zstar L AuxVa Aux2 i Y m pi T Q Welf rho_hh_t gamma_t q R iota mbar G epsilon_t; 
@#for  i in 1:N
    var omega@{i} ;
@#endfor

parameters rho rho_hh delta gamma alpha sigma theta Deltat  Deltaz  psi Upsilon  epsilon phiTR pibar thetaP eta_q share_d rhoTR phiK N_period IP;
@#for  i in 1:N
    parameters z@{i} ;
    parameters varrho@{i} ;
    parameters xi@{i} ;
    parameters bettaA@{i} ;
@#endfor

  
model_parameters
upsilon_HANK

rho      = params(1);
rho_hh   = params(2);
delta    = params(3);
gamma    = params(4);
alpha    = params(5);
psi      = params(6);
sigma    = params(7);
theta    = params(8);
Deltat   = params(9);
Deltaz   = params(10);
Upsilon  = params(11);
thetaP   = params(12);
epsilon  = params(13);
phiTR    = params(14); 
pibar    = params(15);
eta_q    = params(16);
timeless = params(17);
share_d  = params(18);
rhoTR    = params(19);
phiK     = params(20);
N_period = params(22);
IP       = params(23);


@#for  i in 1:N
    z@{i}      = params(end-4*@{N}+@{i});
    varrho@{i} = params(end-3*@{N}+@{i});
    xi@{i}     = params(end-2*@{N}+@{i});
    bettaA@{i} = params(end-1*@{N}+@{i});
@#endfor

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// define model

model;

mbar  = m/((epsilon-1)/epsilon);
phi    = alpha*((1-alpha)/w)^((1-alpha)/(alpha))*mbar^(1/alpha);
w      = (1-alpha)*Z*K(-1)^alpha*L^(-alpha)*mbar;
R      = alpha*Z*K(-1)^(alpha-1)*L^(1-alpha)*(zstar)/(X/(1-(Omega)))*mbar;
K      = D+A;
K      = (iota-delta)*K(-1)*Deltat+K(-1);
A(-1)  = D(-1)/(gamma_t*(1-Omega)-1);
Z      = ((Q*X)/(1-(Omega)))^alpha;
w      = Upsilon*L^psi*C;
Y      = Z*K(-1)^alpha*L^(1-alpha);
T      = (1-mbar)*Y-thetaP/2*pi^2*Y+ q*share_d*A(-1) - G +(iota*q-iota-phiK/2*(iota-delta)^2)*K(-1); % Profits of retailers + price adj cost + dividends - G + profits cap good producer
i      = r(+1)+pi(+1);
A      = Deltat*1/q*((gamma_t*(1-Omega))*(mbar*alpha*Z*K(-1)^(alpha-1)*L^(1-alpha)-R)+R-delta*q-q*rho)*A(-1)+A(-1);
D      = Deltat*((R-delta*q)*D(-1)+w*L-C+T)/q+D(-1);
(C(+1)-C)/(C*Deltat)= r(+1)-rho_hh_t;
r      = (R-delta*q+(q-q(-1))/Deltat)/q;
(r(+1)-(iota-delta))*(q-1-phiK*(iota-delta)) +(q*iota-iota-phiK/2*(iota-delta)^2) = (q(+1)-q)/Deltat-phiK*(iota(+1)-iota)/Deltat ;
(r(+1)-(Y(+1)-Y)/(Y*Deltat))*pi(+1)*thetaP = epsilon_t*(m-(epsilon_t-1)/epsilon_t)+thetaP*(pi(+1)-pi)/Deltat;


%i      = (1-rhoTR*Deltat)*i(-1) + rhoTR*Deltat*(rho_hh_t+phiTR*(pi-pibar)+pibar); % INERTIA 

@#for i in 1:1
(omega@{i}-omega@{i}(-1))/Deltat  =   (1/q*(gamma_t*max(z@{i}*Q*phi-R,0)+R-q*delta-q*rho)-(A-A(-1))/(A(-1)*Deltat))*omega@{i}(-1)
                                       +  bettaA@{i}*omega@{i} + xi@{i}*omega@{i} + xi@{i+1}*omega@{i+1} ;
@#endfor
@#for i in 2:N-1
(omega@{i}-omega@{i}(-1))/Deltat  =   (1/q*(gamma_t*max(z@{i}*Q*phi-R,0)+R-q*delta-q*rho)-(A-A(-1))/(A(-1)*Deltat))*omega@{i}(-1)
                                       +  bettaA@{i}*omega@{i} + varrho@{i-1}*omega@{i-1} + xi@{i+1}*omega@{i+1} ;
@#endfor

@#for i in N:N
(omega@{i}-omega@{i}(-1))/Deltat  =   (1/q*(gamma_t*max(z@{i}*Q*phi-R,0)+R-q*delta-q*rho)-(A-A(-1))/(A(-1)*Deltat))*omega@{i}(-1)
                                      +  bettaA@{i}*omega@{i}  +  varrho@{i-1}*omega@{i-1} +  varrho@{i}*omega@{i} ;
@#endfor

Omega      = (
@#for i in 1:N
        +omega@{i}(-1)*Deltaz*(z@{i}<zstar)
@#endfor
@#for i in 1:N-1
        +((omega@{i}(-1)+0.5*(zstar-z@{i})/Deltaz*(omega@{i+1}(-1)-omega@{i}(-1)))*(zstar-z@{i})/Deltaz-omega@{i}(-1)/2)*Deltaz*(zstar<z@{i+1})*(z@{i}<zstar) 
@#endfor
 );


X          = (
@#for i in 1:N
        +z@{i}*omega@{i}(-1)*Deltaz*(z@{i}>=zstar)
@#endfor
@#for i in 1:N-1
        +((omega@{i+1}(-1)*z@{i+1}+0.5*(z@{i+1}-zstar)/Deltaz*(omega@{i}(-1)*z@{i}-omega@{i+1}(-1)*z@{i+1}))*(z@{i+1}-zstar)/Deltaz-omega@{i+1}(-1)*z@{i+1}/2)*Deltaz*(zstar<z@{i+1})*(z@{i}<zstar) 
@#endfor
        );

AuxVa= Deltat*(Y-C-(rho-share_d)*q*A(-1)-thetaP/2*pi^2*Y-phiK/2*(iota-delta)^2*K(-1))-(K-(1-delta*Deltat)*K(-1)); //Total resource constraint


Aux2= (
    @#for i in 1:N
            +omega@{i}(-1)*Deltaz
    @#endfor
     );

Welf= 0; 


// Law of motion in case of exogenous MIT shock
Q         = exp((1-eta_q*Deltat)*log(Q(-1)));                    // Exogenous TFP
rho_hh_t  = rho_hh_t(-1)*(1-eta_q*Deltat)+eta_q*Deltat*rho_hh;   // HH discount
gamma_t   = gamma_t(-1)*(1-eta_q*Deltat)+eta_q*Deltat*gamma;     // Financial frictions
G         = G(-1)*(1-eta_q*Deltat)+eta_q*Deltat*0;               // Govt spending
epsilon_t = epsilon_t(-1)*(1-eta_q*Deltat)+eta_q*Deltat*epsilon; // Cost push shock



end;


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Provide SS of model conditional on pi
steady_state_model; // provide SS as function of pi without optimal policy

%pi       = 0;
[Omega, phi, w, r, A, X, K, D, Z, C, zstar, L, m, i, Y, T, Aux2,mbar, omega1 ,omega2 ,omega3 ,omega4 ,omega5 ,omega6 ,omega7 ,omega8 ,omega9 ,omega10 ,omega11 ,omega12 ,omega13 ,omega14 ,omega15 ,omega16 ,omega17 ,omega18 ,omega19 ,omega20 ,omega21 ,omega22 ,omega23 ,omega24 ,omega25 ,omega26 ,omega27 ,omega28 ,omega29 ,omega30 ,omega31 ,omega32 ,omega33 ,omega34 ,omega35 ,omega36 ,omega37 ,omega38 ,omega39 ,omega40 ,omega41 ,omega42 ,omega43 ,omega44 ,omega45 ,omega46 ,omega47 ,omega48 ,omega49 ,omega50 ,omega51 ,omega52 ,omega53 ,omega54 ,omega55 ,omega56 ,omega57 ,omega58 ,omega59 ,omega60 ,omega61 ,omega62 ,omega63 ,omega64 ,omega65 ,omega66 ,omega67 ,omega68 ,omega69 ,omega70 ,omega71 ,omega72 ,omega73 ,omega74 ,omega75 ,omega76 ,omega77 ,omega78 ,omega79 ,omega80 ,omega81 ,omega82 ,omega83 ,omega84 ,omega85 ,omega86 ,omega87 ,omega88 ,omega89 ,omega90 ,omega91 ,omega92 ,omega93 ,omega94 ,omega95 ,omega96 ,omega97 ,omega98 ,omega99 ,omega100 ,omega101 ,omega102 ,omega103 ,omega104 ,omega105 ,omega106 ,omega107 ,omega108 ,omega109 ,omega110 ,omega111 ,omega112 ,omega113 ,omega114 ,omega115 ,omega116 ,omega117 ,omega118 ,omega119 ,omega120 ,omega121 ,omega122 ,omega123 ,omega124 ,omega125 ,omega126 ,omega127 ,omega128 ,omega129 ,omega130 ,omega131 ,omega132 ,omega133 ,omega134 ,omega135 ,omega136 ,omega137 ,omega138 ,omega139 ,omega140 ,omega141 ,omega142 ,omega143 ,omega144 ,omega145 ,omega146 ,omega147 ,omega148 ,omega149 ,omega150 ,omega151 ,omega152 ,omega153 ,omega154 ,omega155 ,omega156 ,omega157 ,omega158 ,omega159 ,omega160 ,omega161 ,omega162 ,omega163 ,omega164 ,omega165 ,omega166 ,omega167 ,omega168 ,omega169 ,omega170 ,omega171 ,omega172 ,omega173 ,omega174 ,omega175 ,omega176 ,omega177 ,omega178 ,omega179 ,omega180 ,omega181 ,omega182 ,omega183 ,omega184 ,omega185 ,omega186 ,omega187 ,omega188 ,omega189 ,omega190 ,omega191 ,omega192 ,omega193 ,omega194 ,omega195 ,omega196 ,omega197 ,omega198 ,omega199 ,omega200 ]=main_HANK_ss4(pi,0);
AuxVa     = 0;
Q         = 1;
Welf      = 0;
rho_hh_t  = rho_hh;
gamma_t   = gamma;
q         = 1;
R         = (rho_hh+delta);
iota      = delta;
G         = 0;
epsilon_t = epsilon;
Z         = ((Q*X)/(1-Omega))^alpha;
end;



////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// define planner's problem

planner_objective(ln(C)-Upsilon*((L^(1+psi))/(1+psi)));
ramsey_model(planner_discount=1/(1+rho_hh*Deltat),instruments=(pi));

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Find SS of model with optimal policy

options_.debug=1;
steady(maxit=20,tolf=1e-8);
check;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Calculate transitional dynamic


start = oo_.steady_state;
stop  = oo_.steady_state;

// Define here the MIT shock

%start(22)=start(22)-0.015;            %
%start(23)=start(23)*0.9;              % Credit shock
%start(15) =start(15)-0.07;            % Monetary policy shock (to mimick t0)
%start(15) =start(15)-0.01;            % Monetary policy shock (100bps expansionary shock)
%start(20)=start(20)*0.9;              % TFP  shock
%start(28)=start(28) - 0.01*start(16); % Gov spending shock 1% of GDP
start(29)=start(29)*0.9;              % Cost push shock



initval;
Omega   =start(1);
phi     =start(2);
w       =start(3);
r       =start(4);
A       =start(5);
X       =start(6);
K       =start(7);
D       =start(8);       
Z       =start(9); 
C       =start(10); 
zstar   =start(11);
L       =start(12);
AuxVa   =start(13);
Aux2    =start(14); 
i       =start(15); 
Y       =start(16); 
m       =start(17);
pi      =start(18); 
T       =start(19);
Q       =start(20);
Welf    =start(21);
rho_hh_t=start(22);
gamma_t =start(23);
q       =start(24);
R       =start(25); 
iota    =start(26);
mbar    =start(27);
G       =start(28);
epsilon_t=start(29);



@#for i in 1:N
omega@{i}       =start(@{i+29});
@#endfor
end;

endval;
Omega   =stop(1);
phi     =stop(2);
w       =stop(3);
r       =stop(4);
A       =stop(5);
X       =stop(6);
K       =stop(7);
D       =stop(8);        
Z       =stop(9); 
C       =stop(10); 
zstar   =stop(11);
L       =stop(12);
AuxVa   =stop(13);
Aux2    =stop(14); 
i       =stop(15); 
Y       =stop(16); 
m       =stop(17);
pi      =stop(18); 
T       =stop(19);
Q       =stop(20);
Welf    =stop(21);
rho_hh_t=stop(22);
gamma_t =stop(23);
q       =stop(24);
R       =stop(25);
iota    =stop(26);
mbar    =stop(27);
G       =stop(28);
epsilon_t=stop(29);



@#for i in 1:N
omega@{i}       =stop(@{i+29});
@#endfor
end;


options_.periods = N_period;
perfect_foresight_setup;

if timeless==1
    %by default dynare does timeless
else
    oo_.endo_simul(strmatch('MULT',M_.endo_names(:,:)),1)=0;
end

perfect_foresight_solver(maxit=10,tolx=1e-8,tolf=1e-8);


%-------------------------------------------------------------
%                         GRAPHS
%-------------------------------------------------------------


shock   = epsilon_t; %% Define here the variable that is shocked for graphs


I_dev     = ((iota(1:end).*K(1:end)-delta*K(end))/(delta*K(end))*100);
Y_dev     = (Y(1:end)-Y(end))/Y(end)*100;
TFP       = (Q.*X./(1-Omega)).^alpha ;
r_dev     = (r(1:end)-r(end))*100;
w_dev     = (w(1:end)-w(end))/w(end)*100;
shock_dev = (shock(1:end)-shock(end))/shock(end)*100;



Tn       = N_period*Deltat;               % Time in years - 600 is number of grids
years    = [Deltat:Deltat:Tn];            % Vector of time in years
iniT     = 2;                             % First period 

maxy  = 30;
maxT  = maxy/Deltat;                      % Last year we want for the graphs



figure(12)
subplot(6,3,1),  hold on, plot(years(iniT:maxT),(L(iniT:maxT)-L(end))/L(end)*100,'LineWidth',2)                 , title('L')
subplot(6,3,2),  hold on, plot(years(iniT:maxT),(Y(iniT:maxT)-Y(end))/Y(end)*100,'LineWidth',2)                 , title('Y')
subplot(6,3,3),  hold on, plot(years(iniT:maxT),(pi(iniT:maxT)-pi(end)),'LineWidth',2)                          , title('pi')
subplot(6,3,4),  hold on, plot(years(iniT:maxT),(K(iniT:maxT)-K(end))/K(end)*100,'LineWidth',2)                 , title('K')
subplot(6,3,5),  hold on, plot(years(iniT:maxT),(i(iniT:maxT)-i(end)),'LineWidth',2)                            , title('i')
subplot(6,3,6),  hold on, plot(years(iniT:maxT),((C(iniT:maxT)-C(end))/C(end))*100,'LineWidth',2)               , title('C')
subplot(6,3,7),  hold on, plot(years(iniT:maxT),((m(iniT:maxT)-m(end))/m(end))*100,'LineWidth',2)               , title('m')
subplot(6,3,8),  hold on, plot(years(iniT:maxT),r_dev(iniT:maxT),'LineWidth',2)                                 , title('r')
subplot(6,3,9),  hold on, plot(years(iniT:maxT),I_dev(iniT:maxT),'LineWidth',2)                                 , title('I')
subplot(6,3,10), hold on, plot(years(iniT:maxT),((zstar(iniT:maxT)-zstar(end))/zstar(end)*100),'LineWidth',2)   , title('zstar')
subplot(6,3,11), hold on, plot(years(iniT:maxT),((q(iniT:maxT)-q(end))/q(end)*100),'LineWidth',2)               , title('q')
subplot(6,3,12), hold on, plot(years(iniT:maxT),w_dev(iniT:maxT),'LineWidth',2)                                 , title('w')
subplot(6,3,13), hold on, plot(years(iniT:maxT),((phi(iniT:maxT)-phi(end))/phi(end)*100),'LineWidth',2)         , title('phi')
subplot(6,3,14), hold on, plot(years(iniT:maxT),((A(iniT:maxT)-A(end))/A(end)*100),'LineWidth',2)               , title('A')
subplot(6,3,15), hold on, plot(years(iniT:maxT),(TFP(iniT:maxT)-TFP(end))/TFP(end)*100,'LineWidth',2)           , title('TFP')
subplot(6,3,16), hold on, plot(years(iniT:maxT),(D(iniT:maxT)-D(end))/D(end)*100,'LineWidth',2)                 , title('D')
subplot(6,3,17), hold on, plot(years(iniT:maxT),R(iniT:maxT)-R(end),'LineWidth',2)                              , title('R')
subplot(6,3,18), hold on, plot(years(iniT:maxT),shock_dev(iniT:maxT),'LineWidth',2)                            , title('Shock')



